/*
 * mtalk.h -- set some values for client & servers
 */

/* size of audio buffer */
#define SIZE_MB 1000000		/* that oughta clear the rug */
				/* (goes away when we add memory management */

typedef enum {			/* compression methods */
		ME_FULL 	= 0,	/* full-rate ulaw */
		ME_GSM		= 1,	/* gsm frames */
		ME_UNSUPPORTED		/* error value */
		} METHOD;

char id_compression[] = {	/* prepended to size when transferring */
		'*',		/* use full-rate ulaw */
		'G',		/* use gsm */
		'\0'		/* UNSUPPORTED */
		};
